#!/bin/lua

-- (c) 2005,2006 Robert Shingledecker
-- DSL Calendar View/Edit/Enter Events

require("fltk")
dofile("/etc/init.d/functions5.lua")

function showCurrentEvents()
   target = year..month_num
   for i = 1, selector:size() do
      if string.sub(selector:text(i),1,6)==target then
         selector:show(i)
      else
         selector:hide(i)
      end
   end
end

function saveEvents()
   local eventitems,err = io.open(eventfile,"w")
   if not eventitems then
      print("Cannot open file "..eventfile..":"..err)
      os.exit(1)
   end
   for i = 1, selector:size() do
      if selector:text(i) ~= "" then
         eventitems:write(selector:text(i).."\n")
      end
   end
   eventitems:close()
   saveNeededFlag = nil
end

-- Main

saveNeededFlag = nil
window_width = 280
window_height = 305

eventfile = os.getenv("HOME").."/Events.cal"
currentEvents = os.tmpname()
monthName={"January","February","March","April","May","June","July","August","September","October","November","December"}
month_num = os.date("%m")
year = os.date("%Y")

main = fltk:Fl_Double_Window(305, 280, "Calendar")
main:callback(
function(main)
   if saveNeededFlag then
      response = fltk.fl_choice("Event data has been changed!\nDo you want to exit without saving?","No","Yes",NULL)
      if response == 1 then
         os.exit(0)
      end
   else
      os.exit(0)
   end
end)

month = fltk:Fl_Output(10, 5, 180, 30)
month:box(fltk.FL_FLAT_BOX)
month:color(fltk.FL_BACKGROUND_COLOR)
month:value(monthName[tonumber(month_num)].." "..year)
month_down = fltk:Fl_Button(190, 10, 25, 25, "@<")
month_down:callback(
function(month_down)
   month_num = month_num - 1
   if month_num < 1 then 
      month_num = 12
      year = year - 1
   end
   month_num = string.format("%02u",month_num)
   month:value(monthName[tonumber(month_num)].." "..year)
   showCurrentEvents()
end)
month_up = fltk:Fl_Button(215, 10, 25, 25, "@>")
month_up:callback(
function(month_up)
   month_num = month_num + 1
   if month_num > 12 then 
      month_num = 1
      year = year + 1
   end
   month_num = string.format("%02u",month_num)
   month:value(monthName[tonumber(month_num)].." "..year)
   showCurrentEvents()
end)

viewBtn = fltk:Fl_Button(240, 10, 50, 25, "View")
viewBtn:callback(
function(viewBtn)
   if saveNeededFlag then saveEvents() end
   if fltk.fl_choice("Display Landscape. It is best for printing","No","Yes",NULL) == 1 then
      landscape = ""
   else
      landscape = "-R"
   end
   workfile = os.tmpname()
   os.execute('pscal '..landscape..' '..month_num..' '..year..'  >  '.. workfile)
   os.execute('/usr/local/bin/gvu ' .. workfile) 
   os.remove(workfile)
end)

selector = fltk:Fl_Hold_Browser(5,40,290,195)
selector:callback(
function(selector)
   selected_item = selector:text(selector:value())
end)
if not selector:load(eventfile) then
   print("can't load ", currentEvents)
   os.exit(1)
end
showCurrentEvents()

helpBtn = fltk:Fl_Button(10, 245, 50, 25, "Help")
helpBtn:callback(
function(helpBtn)
   fltk.fl_message(
"Events consists of short text messages up to \n"..
"twenty characters. Each day may have up to six \n"..
"events or a single event up to six lines or a \n"..
"combination thereof.")
end)

editBtn = fltk:Fl_Button(65, 245, 50, 25, "Edit")
editBtn:callback(
function(editBtn)
   if selector:value() == 0 then
      fltk.fl_alert("You must select an event!")
   else
      if selected_item ~= nil then 
         dayNbr = string.sub(selected_item,8,9)
	 editEventWin:label("Edit "..month:value().." "..dayNbr.." Event")
         event_edit:value(string.sub(selected_item,11))
      end
      main:hide()
      editEventWin:show()
   end
end)

newBtn = fltk:Fl_Button(120, 245, 50, 25, "New")
newBtn:callback(
function(newBtn)
   addEventWin:label("Add Event for "..month:value())
   main:hide()
   addEventWin:show()
end)

delBtn = fltk:Fl_Button(175, 245, 50, 25, "Delete")
delBtn:callback(
function(delBtn)
   if selector:value() == 0 then
      fltk.fl_alert("You must select an event!")
   else
      selector:remove(selector:value())
      saveNeededFlag = 1
   end
end)

saveBtn = fltk:Fl_Button(230, 245, 50, 25, "Save")
saveBtn:callback(
function(saveBtn)
   saveEvents()
   os.exit(0)
end)

-------------- addEventWidow ---------------------

addEventWin = fltk:Fl_Window(250,120, " New Event")
addEventWin:callback(
function(addEventWin)
   addEventWin:hide()
   main:show()
end)

day_input = fltk:Fl_Input(60,20,50,25,"Day:")
day_input:value("04")
day_down = fltk:Fl_Button(120,20,25,25)
day_down:label("@<")
day_down:callback(
function(day_down)
   day_num = tonumber(day_input:value()) - 1
   if day_num < 1 then day_num = 31 end
   day_input:value(string.format("%02u",day_num))
end)
day_up = fltk:Fl_Button(145,20,25,25)
day_up:label("@>")
day_up:callback(
function(day_up)
   day_num = tonumber(day_input:value()) + 1
   if day_num > 31 then day_num = 1 end
   day_input:value(string.format("%02u",day_num))
end)

event_input = fltk:Fl_Input(60,50,170,25,"Event:")

ok = fltk:Fl_Button(60,80,70,25,"&OK")
ok:callback(
function(ok)
  if event_input:value() ~= "" then
     newdata = year..month_num..":"..day_input:value()..":"..event_input:value() 
     if selector:value() == 0 then
        selector:add(newdata)
        saveNeededFlag = 1
     else
        selector:insert(selector:value(),newdata)
        saveNeededFlag = 1
     end
  end
  addEventWin:hide()
  main:show()
end)

-------------- editEventWidow ---------------------

editEventWin = fltk:Fl_Window(250,70, "Event Edit")
editEventWin:callback(
function(editEventWin)
   editEventWin:hide()
   main:show()
end)

event_edit = fltk:Fl_Input(60,10,170,25,"Event:")

editok = fltk:Fl_Button(60,40,30,25,"&OK")
editok:callback(
function(editok)
  if event_edit:value() ~= "" then
     newdata = year .. month_num..":"..dayNbr..":"..event_edit:value() 
     selector:text(selector:value(),newdata)
     saveNeededFlag = 1
  end
  editEventWin:hide()
  main:show()
end)

---------------------------------------------------

selector:position(0)
main:resizable(main)
main:show()
Fl:run()
